@@ -20,13 +20,13 @@
 //! #[tokio::main]
 //! async fn main() {
 //!		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-//!		let mut store: Store<String> = Store::new(path).await.unwrap();
+//!		let mut store: Store<u8, String> = Store::new(path).await.unwrap();
 //!
 //!		let stream = futures::stream::iter(
 //!			["zero", "one", "two", "three", "four", "five"]
 //!				.into_iter()
 //!				.enumerate()
-//!				.map(|(i, v)| (i as u64, v.to_string()))
+//!				.map(|(i, v)| (i as u8, v.to_string()))
 //!		);
 //!		store.write_infallible(stream).await.unwrap();
 //!
@@ -47,7 +47,7 @@
 //! #[tokio::main]
 //! async fn main() {
 //!		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-//!		let mut store: Store<String> = Store::new(path).await.unwrap();
+//!		let mut store: Store<u16, String> = Store::new(path).await.unwrap();
 //!
 //!		let stream = futures::stream::iter([
 //!			(200, "two hundred".to_string()),
@@ -66,6 +66,7 @@
 
 use std::cmp::Ordering;
 use std::collections::HashMap;
+use std::hash::Hash;
 use std::io::SeekFrom;
 use std::mem::size_of;
 use std::path::Path;
@@ -84,6 +85,7 @@ use futures::stream::TryStreamExt;
 
 use serde::de::DeserializeOwned;
 use serde::Deserialize;
+use serde::Deserializer;
 use serde::Serialize;
 
 use tokio::fs::File;
@@ -102,6 +104,9 @@ pub use error::Error;
 pub use error::StreamError;
 pub mod stream;
 
+pub trait Key: Copy + Eq + Hash + Ord + std::fmt::Debug + std::fmt::Display + for<'de> Deserialize<'de> + Serialize {}
+impl<T> Key for T where T: Copy + Eq + Hash + Ord + std::fmt::Debug + std::fmt::Display + for<'de> Deserialize<'de> + Serialize {}
+
 const CURRENT_FORMAT_VERSION: u16 = 1;
 
 fn codec() -> impl bincode::Options + Copy /* {{{ */ {
@@ -112,15 +117,15 @@ fn codec() -> impl bincode::Options + Copy /* {{{ */ {
 		.with_native_endian()
 } // }}}
 
-#[derive(Debug, Deserialize, Eq, PartialEq, Serialize)]
-struct IndexEntry {
-	key: u64,
+#[derive(Debug, Eq, PartialEq, Serialize)]
+pub struct IndexEntry<K: Key> {
+	key: K,
 	/// absolute position in the file
 	position: u64
 }
 
-impl IndexEntry {
-	fn new(key: u64, position: u64) -> Self /* {{{ */ {
+impl<K: Key> IndexEntry<K> {
+	fn new(key: K, position: u64) -> Self /* {{{ */ {
 		Self{
 			key,
 			position
@@ -128,18 +133,27 @@ impl IndexEntry {
 	} // }}}
 }
 
-impl Ord for IndexEntry /* {{{ */ {
+impl<K: Key> Ord for IndexEntry<K> /* {{{ */ {
 	fn cmp(&self, other: &Self) -> Ordering {
 		self.key.cmp(&other.key)
 	}
 } // }}}
 
-impl PartialOrd for IndexEntry /* {{{ */ {
+impl<K: Key> PartialOrd for IndexEntry<K> /* {{{ */ {
 	fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
 		Some(self.cmp(other))
 	}
 } // }}}
 
+impl<'de, K: Key> Deserialize<'de> for IndexEntry<K> /* {{{ */ {
+	fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+		let (key, position) = <(K, u64)>::deserialize(deserializer)?;
+		Ok(Self{
+			key, position
+		})
+	}
+} // }}}
+
 #[derive(Debug, Deserialize, Serialize)]
 struct Header {
 	format_version: u16,
@@ -149,20 +163,28 @@ struct Header {
 	index_start_pos: u64
 }
 
-pub struct Store<T: Serialize + DeserializeOwned> /* {{{ */ {
+pub struct Store<K, T> /* {{{ */
+where
+	K: Key,
+	T: Serialize + DeserializeOwned
+{
 	path: PathBuf,
 	file: BufStream<File>,
 	header: Header,
-	index_cache: HashMap<u64, u64, fnv::FnvBuildHasher>,
+	index_cache: HashMap<K, u64, fnv::FnvBuildHasher>,
 	_nothing: std::marker::PhantomData<T>
 } // }}}
 
-impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
+impl<K, T> Store<K, T>
+where
+	K: Key,
+	T: Serialize + DeserializeOwned + std::fmt::Debug,
+{
 	const HEADER_SIZE: usize = size_of::<Header>();
-	const INDEX_ENTRY_SIZE: usize = size_of::<IndexEntry>();
+	const INDEX_ENTRY_SIZE: usize = size_of::<IndexEntry<K>>();
 
 	/// Allocates a new Store containing no values.  Opens a file descriptor to `path`.
-	pub async fn new(path: impl AsRef<Path>) -> Result<Self, Error> /* {{{ */ {
+	pub async fn new(path: impl AsRef<Path>) -> Result<Self, Error<K>> /* {{{ */ {
 		let fd = tokio::fs::OpenOptions::new()
 			.read(true)
 			.write(true)
@@ -194,29 +216,30 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 		Self::HEADER_SIZE as u64
 	} // }}}
 
-	async fn get_header(&mut self) -> Result<Header, Error> /* {{{ */ {
+	async fn get_header(&mut self) -> Result<Header, Error<K>> /* {{{ */ {
 		let mut buf = [0u8; size_of::<Header>()];
 		self.file.seek(SeekFrom::Start(0)).await?;
 		self.file.read(&mut buf).await?;
 		Ok(deserialize(&buf)?)
 	} // }}}
 
-	async fn read_index_entry(&mut self) -> Result<IndexEntry, Error> /* {{{ */ {
-		let mut buf = [0u8; size_of::<IndexEntry>()];
-		self.file.read(&mut buf).await?;
-		Ok(deserialize(&buf)?)
+	async fn read_index_entry(&mut self) -> Result<IndexEntry<K>, Error<K>> /* {{{ */ {
+		// This will fail if size_of::<K>() > 128
+		let mut buf = [0u8; 192];
+		let count = self.file.read(&mut buf).await?;
+		Ok(deserialize(&buf[..count])?)
 	} // }}}
 
 	#[cfg_attr(feature = "full-instrumentation", instrument(err, level = "trace", skip(self), fields(indexkv.path = %self.path.display())))]
-	async fn get_index_entry(&mut self, i: u64) -> Result<IndexEntry, Error> /* {{{ */ {
+	async fn get_index_entry(&mut self, i: u64) -> Result<IndexEntry<K>, Error<K>> /* {{{ */ {
 		let pos = self.index_start_position() + (i * Self::INDEX_ENTRY_SIZE as u64);
 		self.file.seek(SeekFrom::Start(pos)).await?;
 		let entry = self.read_index_entry().await?;
 		Ok(entry)
 	} // }}}
 
-	#[instrument(err, level = "debug", skip(self), fields(indexkv.path = %self.path.display()))]
-	async fn find_index_entry(&mut self, key: u64) -> Result<Option<IndexEntry>, Error> /* {{{ */ {
+	#[cfg_attr(feature = "full-instrumentation", instrument(err, level = "debug", skip(self), fields(indexkv.path = %self.path.display())))]
+	async fn find_index_entry(&mut self, key: K) -> Result<Option<IndexEntry<K>>, Error<K>> /* {{{ */ {
 		let mut size = self.header.pair_count;
 		let mut left = 0;
 		let mut right = size;
@@ -233,7 +256,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 		Ok(None)
 	} // }}}
 
-	async fn read_value(&mut self) -> Result<Option<T>, Error> /* {{{ */ {
+	async fn read_value(&mut self) -> Result<Option<T>, Error<K>> /* {{{ */ {
 		let size = match AsyncReadBytesExt::read_u32::<NativeEndian>(&mut self.file).await {
 			Ok(v) => v as usize,
 			Err(e) => match e.kind() {
@@ -254,7 +277,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 		Ok(Some(deserialize(&buf)?))
 	} // }}}
 
-	async fn read_value_at(&mut self, position: u64) -> Result::<T, Error> /* {{{ */ {
+	async fn read_value_at(&mut self, position: u64) -> Result::<T, Error<K>> /* {{{ */ {
 		self.file.seek(SeekFrom::Start(position)).await?;
 		match self.read_value().await? {
 			Some(v) => Ok(v),
@@ -265,7 +288,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	/// Reads the header and checks to see whether or not the store is valid; at this time, this
 	/// means making sure the format is fully up-to-date and writing completed.
 	#[inline]
-	pub async fn is_valid(&mut self) -> Result<bool, Error> /* {{{ */ {
+	pub async fn is_valid(&mut self) -> Result<bool, Error<K>> /* {{{ */ {
 		Ok(
 			self.header.format_version == CURRENT_FORMAT_VERSION &&
 			self.header.completed_writing
@@ -278,7 +301,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	//    to the Store.  This will, naturally, ruin everything.
 	#[instrument(err, level = "debug", skip(self), fields(indexkv.path = %self.path.display()))]
 	#[inline]
-	pub async fn stream_values(&self) -> Result<impl Stream<Item=Result<T, Error>>, Error> /* {{{ */ {
+	pub async fn stream_values(&self) -> Result<impl Stream<Item=Result<T, Error<K>>>, Error<K>> /* {{{ */ {
 		let mut file = tokio::fs::OpenOptions::new()
 			.read(true)
 			.write(false)
@@ -286,7 +309,10 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 			.append(false)
 			.open(&self.path)
 			.await?;
-		file.seek(SeekFrom::Start(self.data_start_position())).await?;
+		let pos = SeekFrom::Start(self.data_start_position());
+		println!("pos = {:?}", pos);
+		file.seek(pos).await?;
+		println!("pair count = {}", self.header.pair_count);
 		let stream = stream::ValueStream::new(file).take(self.header.pair_count as usize);
 		Ok(stream)
 	} // }}}
@@ -296,7 +322,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	/// Store.
 	#[cfg_attr(feature = "full-instrumentation", instrument(err, level = "debug", skip(self), fields(indexkv.path = %self.path.display())))]
 	#[inline]
-	pub async fn get(&mut self, key: u64) -> Result<T, Error> /* {{{ */ {
+	pub async fn get(&mut self, key: K) -> Result<T, Error<K>> /* {{{ */ {
 		let position = match self.index_cache.get(&key) {
 			Some(v) => *v,
 			None => {
@@ -315,7 +341,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	/// disk seeking back and forth.  Does _not_ return an error when `key` does not exist; calling
 	/// code should check for `Some` vs `None` in the resulting `HashMap` for fallibility.
 	#[instrument(err, level = "debug", skip(self), fields(indexkv.path = %self.path.display()))]
-	pub async fn get_many(&mut self, keys_in: &[u64]) -> Result<HashMap<u64, T>, Error> /* {{{ */ {
+	pub async fn get_many(&mut self, keys_in: &[K]) -> Result<HashMap<K, T>, Error<K>> /* {{{ */ {
 		let mut results = HashMap::with_capacity(keys_in.len());
 		let mut keys = Vec::new();
 		for key in keys_in {
@@ -353,9 +379,9 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	//   * Write the sorted index down to the file
 	//   * Seek to the beginning of the file and write down the header, with completed_writing = true
 	#[instrument(err, level = "info", skip(self, input_stream, writer), fields(indexkv.path = %self.path.display()))]
-	async fn write_to<S, W>(&mut self, mut input_stream: S, writer: &mut W) -> Result<(), StreamError<S::Error>> /* {{{ */
+	async fn write_to<S, W>(&mut self, mut input_stream: S, writer: &mut W) -> Result<(), StreamError<S::Error, K>> /* {{{ */
 	where
-		S: TryStream<Ok=(u64, T)> + Unpin,
+		S: TryStream<Ok=(K, T)> + Unpin,
 		W: AsyncWrite + AsyncSeek + Unpin,
 		S::Error: std::error::Error
 	{
@@ -427,11 +453,11 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	///		#[error("I/O error")]
 	///		IO(#[from] io::Error),
 	///		#[error("cache error")]
-	///		Cache(#[from] indexkv::Error)
+	///		Cache(#[from] indexkv::Error<u64>)
 	/// }
 	///
-	/// impl From<StreamError<MyError>> for MyError {
-	///		fn from(other: StreamError<MyError>) -> Self {
+	/// impl From<StreamError<MyError, u64>> for MyError {
+	///		fn from(other: StreamError<MyError, u64>) -> Self {
 	///			match other {
 	///				StreamError::Caller(e) => e,
 	///				StreamError::Internal(e) => MyError::from(e)
@@ -441,7 +467,7 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	///
 	/// async fn write() -> Result<(), MyError> {
 	///		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-	///		let mut store: Store<String> = Store::new(path).await.unwrap();
+	///		let mut store: Store<u64, String> = Store::new(path).await.unwrap();
 	///
 	///		let stream = futures::stream::iter(
 	///			vec![Ok("zero"), Ok("one"), Err(io::Error::from(io::ErrorKind::BrokenPipe)), Ok("two")]
@@ -466,9 +492,9 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	///	}
 	/// ```
 	#[inline]
-	pub async fn write<S>(&mut self, input_stream: S) -> Result<(), StreamError<S::Error>> /* {{{ */
+	pub async fn write<S>(&mut self, input_stream: S) -> Result<(), StreamError<S::Error, K>> /* {{{ */
 	where
-		S: TryStream<Ok=(u64, T)> + Unpin,
+		S: TryStream<Ok=(K, T)> + Unpin,
 		S::Error: std::error::Error
 	{
 		self.index_cache.clear();
@@ -479,11 +505,11 @@ impl<T: Serialize + DeserializeOwned + std::fmt::Debug> Store<T> {
 	/// For cases where the incoming data stream is infallible, this allows you to avoid that
 	/// portion of error handling.
 	#[inline]
-	pub async fn write_infallible<S>(&mut self, input_stream: S) -> Result<(), Error> /* {{{ */
+	pub async fn write_infallible<S>(&mut self, input_stream: S) -> Result<(), Error<K>> /* {{{ */
 	where
-		S: Stream<Item=(u64, T)> + Unpin
+		S: Stream<Item=(K, T)> + Unpin
 	{
-		self.write(input_stream.map(Result::<_, Error>::Ok)).await.map_err(|e| match e {
+		self.write(input_stream.map(Result::<_, Error<K>>::Ok)).await.map_err(|e| match e {
 			StreamError::Internal(e) => e,
 			_ => unreachable!()
 		})
@@ -518,8 +544,8 @@ mod tests {
 	async fn constant_u8_1() /* {{{ */ {
 		let values = vec![42u8];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<u8> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u64, v) }))).await.unwrap();
+		let mut store: Store<u8, u8> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u8, v) }))).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 42u8);
@@ -529,8 +555,8 @@ mod tests {
 	async fn constant_u8_2() /* {{{ */ {
 		let values = vec![11u8, 32u8];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<u8> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u64, v) }))).await.unwrap();
+		let mut store: Store<i16, u8> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i16, v) }))).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 11u8);
@@ -542,8 +568,8 @@ mod tests {
 	async fn constant_u8_3() /* {{{ */ {
 		let values = vec![47u8, 53u8, 128u8];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<u8> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u64, v) }))).await.unwrap();
+		let mut store: Store<u32, u8> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u32, v) }))).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 47u8);
@@ -557,8 +583,8 @@ mod tests {
 	async fn constant_u16_1() /* {{{ */ {
 		let values = vec![42u16];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<u16> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u64, v) }))).await.unwrap();
+		let mut store: Store<i128, u16> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i128, v) }))).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 42u16);
@@ -568,8 +594,8 @@ mod tests {
 	async fn constant_u16_2() /* {{{ */ {
 		let values = vec![11u16, 32u16];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<u16> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u64, v) }))).await.unwrap();
+		let mut store: Store<i32, u16> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i32, v) }))).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 11u16);
@@ -583,11 +609,11 @@ mod tests {
 		values.extend((0..255_u8).map(|i| FixedWidth::new2(i)));
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		let mut store: Store<u16, FixedWidth> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u16, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
-		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
+		let mut indices = (0..values.len() as u16).collect::<Vec<u16>>();
 		for _ in 1..16 {
 			indices.shuffle(&mut rng);
 			for i in indices.iter() {
@@ -622,11 +648,11 @@ mod tests {
 		// }}}
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<Struct> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		let mut store: Store<i64, Struct> = Store::new(path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as i64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
-		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
+		let mut indices = (0..values.len() as i64).collect::<Vec<i64>>();
 		for _ in 1..16 {
 			indices.shuffle(&mut rng);
 			for i in indices.iter() {
@@ -662,11 +688,11 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		for _ in 1..4 {
-			let mut store: Store<Struct> = Store::new(&path).await.unwrap();
-			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+			let mut store: Store<u128, Struct> = Store::new(&path).await.unwrap();
+			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u128, v)))).await.unwrap();
 
 			let mut rng = rand::thread_rng();
-			let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
+			let mut indices = (0..values.len() as u128).collect::<Vec<u128>>();
 			for _ in 1..16 {
 				indices.shuffle(&mut rng);
 				for i in indices.iter() {
@@ -682,8 +708,8 @@ mod tests {
 		let values = (0..5_u8).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().keep().unwrap().1;
-		let mut store: Store<FixedWidth> = Store::new(&path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		let mut store: Store<i8, FixedWidth> = Store::new(&path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as i8, v)))).await.unwrap();
 
 		let mut stream = store.stream_values().await.unwrap().enumerate();
 		while let Some((i, result)) = stream.next().await {
@@ -703,7 +729,7 @@ mod tests {
 		values.extend((0..255_u8).map(|i| FixedWidth::new2(i)));
 
 		let path = tempfile::NamedTempFile::new().unwrap().keep().unwrap().1;
-		let mut store: Store<FixedWidth> = Store::new(&path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut stream = store.stream_values().await.unwrap().enumerate();
@@ -724,10 +750,25 @@ mod tests {
 		values.extend((0..255_u8).map(|i| FixedWidth::new2(i)));
 
 		let path = tempfile::NamedTempFile::new().unwrap().keep().unwrap().1;
+		let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		drop(store);
 		for iteration in 1..4 {
-			println!(">>> {}", iteration);
-			let mut store: Store<FixedWidth> = Store::new(&path).await.unwrap();
-			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+			println!(">>> 1-{}", iteration);
+			let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
+
+			let mut stream = store.stream_values().await.unwrap().enumerate();
+			while let Some((i, result)) = stream.next().await {
+				let value = result.unwrap();
+				assert_eq!(value, values[i]);
+			}
+
+			assert_eq!(store.get(0).await.unwrap(), values[0]);
+			assert_eq!(store.get(1).await.unwrap(), values[1]);
+		}
+		for iteration in 1..4 {
+			println!(">>> 2-{}", iteration);
+			let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
 
 			let mut stream = store.stream_values().await.unwrap().enumerate();
 			while let Some((i, result)) = stream.next().await {
@@ -746,7 +787,7 @@ mod tests {
 		let values = (0..5).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -761,7 +802,7 @@ mod tests {
 		let values = (0..5).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
@@ -780,7 +821,7 @@ mod tests {
 		let values = (0..13).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
@@ -799,7 +840,7 @@ mod tests {
 		let values = (0..41).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
@@ -818,7 +859,7 @@ mod tests {
 		let values = (0..47).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
@@ -837,7 +878,7 @@ mod tests {
 		let values = (0..53).map(|i| FixedWidth::new(i)).collect::<Vec<_>>();
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
@@ -857,7 +898,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		for _ in 1..2 {
-			let mut store: Store<FixedWidth> = Store::new(&path).await.unwrap();
+			let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
 			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 			let mut rng = rand::thread_rng();
@@ -878,7 +919,7 @@ mod tests {
 		values.extend((0..255_u8).map(|i| FixedWidth::new2(i)));
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
-		let mut store: Store<FixedWidth> = Store::new(path).await.unwrap();
+		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
 		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
 
 		let mut rng = rand::thread_rng();
